
metropolis <- function(x, ...) UseMethod("metropolis")

metropolis.default <- function(func, state, niter, env=environment(),
    control=list(scaling=1), ...) {

    if (! is.numeric(state)) stop("state must be numeric")
    if (! exists(".Random.seed")) runif(1)
    saveseed <- .Random.seed
    savestate <- state
    extra <- pairlist(...)
    out.time <- system.time(
        out <- .Call("metropolis", func, state, extra, env, niter, control)
    )
    out$seed <- saveseed
    out$initial <- savestate
    out$time <- out.time
    out$final <- state
    out$control <- control
    out$density <- func
    out$extra <- extra
    out$env <- env
    class(out) <- "metropolis"
    return(out)
}

metropolis.metropolis <- function(o, niter) {
    state <- o$final
    func <- o$density
    if (! exists(".Random.seed")) runif(1)
    saveseed <- .Random.seed
    savestate <- state
    out.time <- system.time(
        out <- .Call("metropolis", func, state, o$extra, o$env, niter, o$control)
    )
    out$seed <- saveseed
    out$initial <- savestate
    out$time <- out.time
    out$final <- state
    out$control <- o$control
    out$density <- func
    out$extra <- o$extra
    out$env <- o$env
    class(out) <- "metropolis"
    return(out)
}

